@@ -1,4 +1,4 @@
-#!/usr/bin/python2
+#!/usr/bin/python3
 
 r'''A simple parser for vnlog data
 
@@ -6,7 +6,7 @@ Synopsis:
 
     import vnlog
     for d in vnlog.vnlog(f):
-        print d['time'],d['height']
+        print(d['time'],d['height'])
 
 Vnlog is simple, and you don't NEED a parser to read it, but this makes it a bit
 nicer.
@@ -26,7 +26,7 @@ This module provides three different ways to parse vnlog
 
    import vnlog
    for d in vnlog.vnlog(filename_or_fileobject):
-       print d['time'],d['height']
+       print(d['time'],d['height'])
 
    Null data values are represented as None
 
@@ -41,7 +41,7 @@ This module provides three different ways to parse vnlog
        d = parser.values_dict()
        if not d:
            continue
-       print d['time'],d['height']
+       print(d['time'],d['height'])
 
 Most of the time you'd use options 1 or 2 above. Option 3 is the most general,
 but also the most verbose
@@ -63,7 +63,7 @@ class vnlog:
 
        import vnlog
        for d in vnlog.vnlog(filename_or_fileobject):
-           print d['time'],d['height']
+           print(d['time'],d['height'])
 
        Null data values are represented as None
 
@@ -78,7 +78,7 @@ class vnlog:
            d = parser.values_dict()
            if not d:
                continue
-           print d['time'],d['height']
+           print(d['time'],d['height'])
 
     '''
 
@@ -123,17 +123,16 @@ class vnlog:
         self._values      = None
         self._values_dict = None
 
-        if re.match('^\s*$', l):
-            # empty line
+        if not hasattr(self, 're_hard_comment'):
+            self.re_hard_comment = re.compile('^\s*(?:#[#!]|#\s*$|$)')
+            self.re_soft_comment = re.compile('^\s*#\s*(.*?)\s*$')
+
+        if self.re_hard_comment.match(l):
+            # empty line or hard comment.
             # no data, no error
             return True
 
-        if re.match('^\s*#[!#]', l):
-            # comment
-            # no data, no error
-            return True
-
-        m = re.match('^#\s*(.*?)\s*$', l)
+        m = self.re_soft_comment.match(l)
         if m:
             if self._keys is not None:
                 # already have legend, so this is just a comment
@@ -150,8 +149,7 @@ class vnlog:
             raise Exception("Got dataline before legend")
 
         # strip leading, trailing whitespace
-        m = re.match('\s*(.*?)\s*$', l)
-        l = m.group(1)
+        l = l.strip()
 
         self._values = [ None if x == '-' else x for x in l.split()]
         if len(self._values) != len(self._keys):
